;**************************************************************************************************************************************************
;   LIBRARY:      GESTIONE LCD 4x20, 16x2 IN MODALITA' 4 bit
;   FILE:         LCD_4bit.asm
;   LAST REV:     1.2
;	AUTHOR:       Ivo Colleoni
;  	DEVICE:       PIC18F2550
;	CREATED:	  08/02/2013
;	UPDATED:	  21/05/2013
;
;	DESCRIP:      Gestisce la comunicazione con il display LCD
;
;   FUNCTION:     LCD_INIT
;                 LCD_SEND_CODE
;                 LCD_SEND_DATA
;                 LCD_MOVE_LOC_Rx
;                 LCD_SEND_NUMBER
;                 RESET_VAR_LCD
;
;   PRECONDITION: inserire all'inizio del programma principale:
;                 #DEFINE     PORT_LCD       LATx (PORTx)       per definisce au quale porta sono collegati i 4 bit data (da bit 0 a bit 3)
;                 #DEFINE     LCD_E          LATx,x (PORTx,x)	per definire dove  collegato il pin E dell'LCD
;                 #DEFINE     LCD_RS         LATx,x (PORTx,x)   per definire dove  collegato il pin RS dell'LCD
;                 #CONSTANT   LCD_UNUSED =   b'11000000'        per indicare quali uscite della porta non sono usate dai dati (e da E e RS se consecutivi a data)
;                 Configurazione standard su LATB da RB0 a RB5
;                 5  4  3  2  1  0
;                 E  RS D7 D6 D5 D4
;                 includere alla fine del file principale anche la libreria dei ritardi a seconda del clock usato.
;--------------------------------------------------------------------------------------------------------------------------------------------------
;   Change History:
;  	Rev.    Date                Changes
;   1.2     21/05/2013          - implementato controllo dei bit non utilizzati anche in fase di inizializzazione
;   1.1     04/04/2013          - implementato controllo dei bit non utilizzati sulla porta dedicata all'LCD mediante maschera costante
;   1.0     08/02/2013          - prima build
;
;**************************************************************************************************************************************************
;Dichiarazione variabili Memoria allocata da 0x00 a 0x03
;**************************************************************************************************************************************************
                    CBLOCK              0x00
                    data_t      
                    code_t      
                    rotate
                    lcd_unused
                    ENDC 
;**************************************************************************************************************************************************
;Dichiarazione costanti
;**************************************************************************************************************************************************
                    CONSTANT LCD_CLEAR              = b'00000001'                       ;Cancella tutto l'LCD
                    CONSTANT LCD_HOME               = b'00000010'                       ;Riporta il cursore all'inizio della prima riga
                    CONSTANT LCD_OFF                = b'00001000'                       ;Spegne l'LCD
                    CONSTANT LCD_ON                 = b'00001100'                       ;Accemde l'LCD
                    CONSTANT LCD_BLINK_ON           = b'00001101'                       ;Accende il cursore lampeggiante
                    CONSTANT LCD_BLINK_OFF          = b'00001100'                       ;Spegne il cursore lampeggiante (equivale a LCD_ON)
                    CONSTANT LCD_CURSOR_ON          = b'00001110'                       ;Accende la linea cursore
                    CONSTANT LCD_CURSOR_OFF         = b'00001100'                       ;Spegne il cursore lampeggiante (equivale a LCD_ON)
;
;**************************************************************************************************************************************************
;   FUNCTION:       LCD_INIT
;   Overview:       sottoprogramma per inizializzare il display: ATTENZIONE, in questo ciclo non  vengono gestiti i pin non usati dall'LCD, quindi veranno
;                   sovrascritti
;**************************************************************************************************************************************************
;                                       E/RS/D7/D6/D5/D4
LCD_INIT    		movf                PORT_LCD,W                                      ;leggo il valore attuale della porta dedicata al display
                    andlw               LCD_UNUSED                                      ;e salvo i valori dei bit non utilizzati per l'LCD
                    movwf               lcd_unused

                    movlw               b'00000011'                                     ;reset software LCD
                    addwf               lcd_unused,W                                    ;aggiungo i valori precedentemente salvati dei bit non utilizzati per l'LCD
					movwf               PORT_LCD
					bsf                 LCD_E
					bcf                 LCD_E
					call                DELAY_2ms                                       ;delay 6ms
                    call                DELAY_2ms
                    call                DELAY_2ms

					movlw               b'00000011'
                    addwf               lcd_unused,W                                    ;aggiungo i valori precedentemente salvati dei bit non utilizzati per l'LCD
					movwf               PORT_LCD
					bsf                 LCD_E
					bcf                 LCD_E
					call                DELAY_2ms                                       ;delay 2ms, basterebbe meno ma da prove fatte ci sono problemi

					movlw               b'00000011'
					addwf               lcd_unused,W                                    ;aggiungo i valori precedentemente salvati dei bit non utilizzati per l'LCD
                    movwf               PORT_LCD
					bsf                 LCD_E
					bcf                 LCD_E
					call                DELAY_2ms

                    movlw               b'00000010'                                     ;Setto la modalit 4 bit   (togliere??)
                    addwf               lcd_unused,W                                    ;aggiungo i valori precedentemente salvati dei bit non utilizzati per l'LCD
                    movwf               PORT_LCD
					bsf                 LCD_E
					bcf                 LCD_E
					call                DELAY_2ms

					movlw               b'00101000'                                     ;Setto ancora la modalit ,il numero di righe e la grandezza font
					call                LCD_SEND_CODE                                   ;0/0/1/0(4bit)/1(2 linee)/0(font 5x7)/0/0


					movlw               b'00001000'                                     ;Spengo il display
					call                LCD_SEND_CODE
					movlw               b'00000001'                                     ;pulizia display
					call                LCD_SEND_CODE
					
					movlw               b'00000110'                                     ;Setto la direzione del cursore
                  	call                LCD_SEND_CODE
					movlw               b'00001100'                                     ;Accendo display
					call                LCD_SEND_CODE

					return
;
;**************************************************************************************************************************************************
;   FUNCTION:       LCD_SEND_CODE
;   Input:          WREG: dato da inviare
;                   LCD_UNUSED: maschera per non sovrascrivere i dati sui bit non utilizzati dall'LCD
;   Overview:       Manda i codici di funzionamento all'LCD (trasforma i dati da 8 bit a 4 bit)
;**************************************************************************************************************************************************
;
LCD_SEND_CODE		movwf               code_t                                          ;salvo il dato il una variabile temporanea
					movwf               rotate                                          ;e in una variabile per la rotazione

                    movf                PORT_LCD,W                                      ;leggo il valore attuale della porta dedicata al display
                    andlw               LCD_UNUSED                                      ;e salvo i valori dei bit non utilizzati per l'LCD
                    movwf               lcd_unused

                    rrncf               rotate,F                                        ;ruoto il dato 4 volte per portare i 4 bit piu alti
					rrncf               rotate,F                                        ;nei 4 bit piu bassi
					rrncf               rotate,F
					rrncf               rotate,F
					movf                rotate,W
					andlw               b'00001111'                                     ;maschero i 4 bit piu bassi
                    addwf               lcd_unused,W                                    ;aggiungo i valori precedentemente salvati dei bit non utilizzati per l'LCD
					movwf               PORT_LCD                                        ;mando il dato al display
					bsf                 LCD_E                                      		;abilito la scrittura del dato
					bcf                 LCD_E
					call                DELAY_2ms                                       ;attendo

					movf                code_t,W                                        ;riprendo la variabile originale
					andlw               b'00001111'                                     ;maschero i 4 bit piu bassi
                    addwf               lcd_unused,W                                    ;aggiungo i valori precedentemente salvati dei bit non utilizzati per l'LCD
                    movwf               PORT_LCD                                        ;mando il dato al display
					bsf                 LCD_E                                      		;abilito la scrittura del dato
					bcf                 LCD_E
					call                DELAY_2ms                                       ;attendo

					return
;
;**************************************************************************************************************************************************
;   FUNCTION:       LCD_SEND_DATA
;   Input:          WREG: dato da inviare
;                   LCD_UNUSED: maschera per non sovrascrivere i dati sui bit non utilizzati dall'LCD
;   Overview:       Manda i caratteri all'LCD (trasforma i dati da 8 bit a 4 bit)
;**************************************************************************************************************************************************
;
LCD_SEND_DATA		movwf               data_t                                          ;salvo il dato il una variabile temporanea
					movwf               rotate                                          ;e in una variabile per la rotazione

                    movf                PORT_LCD,W                                      ;leggo il valore attuale della porta dedicata all'LCD
                    andlw               LCD_UNUSED                                      ;e salvo i valori dei bit non utilizzati per l'LCD
                    movwf               lcd_unused

                    rrncf               rotate,F                                        ;ruoto il dato 4 volte per portare i 4 bit piu alti
					rrncf               rotate,F                                        ;nei 4 bit piu bassi
					rrncf               rotate,F
					rrncf               rotate,F
					movf                rotate,W
					andlw               b'00001111'                                     ;maschero i 4 bit piu bassi
                    addwf               lcd_unused,W                                    ;aggiungo i valori precedentemente salvati dei bit non utilizzati per l'LCD
					movwf               PORT_LCD                                        ;mando il dato al display
					bsf					LCD_RS		                                    ;Abilito ance RS per indicare che mando un dato da esporre, e non una funzione
					bsf                 LCD_E                                      		;abilito la scrittura del dato
                    bcf                 LCD_E
					call                DELAY_180us                                     ;attendo

					movf                data_t,W                                        ;Riprendo la variabile originale
					andlw               b'00001111'                                     ;Maschero i 4 bit piu bassi
                    addwf               lcd_unused,W                                    ;aggiungo i valori precedentemente salvati dei bit non utilizzati per l'LCD
					movwf               PORT_LCD                                        ;mando il dato al display
					bsf					LCD_RS		                                    ;Abilito ance RS per indicare che mando un dato da esporre, e non una funzione
					bsf                 LCD_E                                      		;abilito la scrittura del dato
					bcf                 LCD_E
                    call                DELAY_180us                                     ;attendo

					return
;
;**************************************************************************************************************************************************
;   FUNCTION:       LCD_MOVE_LOC_Rx
;   Input:          WREG: posizione sulla riga
;                   Rx x:riga desiderata
;   Overview:       Sposta il cursore (visibile o invisibile) alla posizione contenuta in W della riga Rx dove x  il numero delle riga
;**************************************************************************************************************************************************
;
LCD_MOVE_LOC_R1		addlw               b'10000000'                                     ;Codice per indicare il cursore a inizio prima riga
					call                LCD_SEND_CODE
					return

LCD_MOVE_LOC_R2		addlw               b'11000000'                                     ;Codice per indicare il cursore a inizio seconda riga
					call                LCD_SEND_CODE
					return

LCD_MOVE_LOC_R3 	addlw               b'10010100'                                     ;Codice per indicare il cursore a inizio terza riga
					call                LCD_SEND_CODE
					return

LCD_MOVE_LOC_R4 	addlw               b'11010100'                                     ;Codice per indicare il cursore a inizio quarta riga
					call                LCD_SEND_CODE
					return
;
;**************************************************************************************************************************************************
;   FUNCTION:       SEND_NUMBER
;   Input:          WREG: numero da scrivere sull'LCD
;   Overview:       somma il valore in W con il valore standard per i numeri dei primi 4 bit e manda il dato
;**************************************************************************************************************************************************
;
LCD_SEND_NUMBER		addlw               b'00110000'                                     ;Aggiungo il valore standard dei primi 4 bit per un numero
					call                LCD_SEND_DATA
					return
;
;**************************************************************************************************************************************************
;   FUNCTION:       RESET_VAR_LCD
;   Overview:       Resetta tutte le variabili utilizzate nella gestioe dell'LCD
;**************************************************************************************************************************************************
;
RESET_VAR_LCD       clrf                data_t
                    clrf                code_t
                    clrf                rotate
                    clrf                lcd_unused
                    return
;**************************************************************************************************************************************************
;TABELLA TRANSCODIFICA CARATTERI STANDARD, per mandare un carattere dal programma principale basta caricare in W "c_carattere desiderato"
;**************************************************************************************************************************************************
;
					CONSTANT			c_			= h'20'
					CONSTANT			c_.			= h'2E'
					CONSTANT			c_A			= h'41'
					CONSTANT			c_B			= h'42'
					CONSTANT			c_C			= h'43'
					CONSTANT			c_D			= h'44'
					CONSTANT			c_E			= h'45'
					CONSTANT			c_F			= h'46'
					CONSTANT			c_G			= h'47'
					CONSTANT			c_H			= h'48'
					CONSTANT			c_I			= h'49'
					CONSTANT			c_J			= h'4A'
					CONSTANT			c_K			= h'4B'
					CONSTANT			c_L			= h'4C'
					CONSTANT			c_M			= h'4D'
					CONSTANT			c_N			= h'4E'
					CONSTANT			c_O			= h'4F'
					CONSTANT			c_P			= h'50'
					CONSTANT			c_Q			= h'51'
					CONSTANT			c_R			= h'52'
					CONSTANT			c_S			= h'53'
					CONSTANT			c_T			= h'54'
					CONSTANT			c_U			= h'55'
					CONSTANT			c_V			= h'56'
					CONSTANT			c_W			= h'57'
					CONSTANT			c_X			= h'58'
					CONSTANT			c_Y			= h'59'
					CONSTANT			c_Z			= h'5A'
					CONSTANT			c_a			= h'61'
					CONSTANT			c_b			= h'62'
					CONSTANT			c_c			= h'63'
					CONSTANT			c_d			= h'64'
					CONSTANT			c_e			= h'65'
					CONSTANT			c_f			= h'66'
					CONSTANT			c_g			= h'67'
					CONSTANT			c_h			= h'68'
					CONSTANT			c_i			= h'69'
					CONSTANT			c_j			= h'6A'
					CONSTANT			c_k			= h'6B'
					CONSTANT			c_l			= h'6C'
					CONSTANT			c_m			= h'6D'
					CONSTANT			c_n			= h'6E'
					CONSTANT			c_o			= h'6F'
					CONSTANT			c_p			= h'70'
					CONSTANT			c_q			= h'71'
					CONSTANT			c_r			= h'72'
					CONSTANT			c_s			= h'73'
					CONSTANT			c_t			= h'74'
					CONSTANT			c_u			= h'75'
					CONSTANT			c_v			= h'76'
					CONSTANT			c_w			= h'77'
					CONSTANT			c_x			= h'78'
					CONSTANT			c_y			= h'79'
					CONSTANT			c_z			= h'7A'